"""Common types for the jbutils package"""

from collections.abc import Callable
from typing import TypeVar, Optional, Pattern, Sequence, Literal

# General Typing
T = TypeVar("T")
OptStr = Optional[str]
OptInt = Optional[int]
OptFloat = Optional[float]
OptDict = Optional[dict]
OptList = Optional[list]
Opt = Optional[T]

# Function Types
Predicate = Callable[[T], bool]
Function = Callable[..., None]
TFunction = Callable[..., T]

# Other
Patterns = Sequence[str | Pattern[str]]
DataPathList = list[str] | list[str | int] | list[int]
DataPath = DataPathList | str | int

SubReturn = Literal["out", "err", "both"]
""" String literal type representing the output choices for cmdx """

__all__ = [
    "OptStr",
    "OptInt",
    "OptFloat",
    "OptDict",
    "OptList",
    "Opt",
    "Patterns",
    "Predicate",
    "Function",
    "SubReturn",
    "TFunction",
    "T",
]



"""Package for general utility modules"""

from jbutils.utils.utils import (
    cmdx,
    copy_to_clipboard,
    get_nested,
    parse_csv_line,
    pretty_print,
    read_file,
    set_encoding,
    set_nested,
    set_yaml_indent,
    to_csv_line,
    write_file,
)

__all__ = [
    "cmdx",
    "copy_to_clipboard",
    "get_nested",
    "parse_csv_line",
    "pretty_print",
    "read_file",
    "set_encoding",
    "set_nested",
    "set_yaml_indent",
    "to_csv_line",
    "write_file",
]



"""CLI Testing tool for checking local functionality"""

import json
import os

from dataclasses import dataclass

from ptpython import embed

from jbutils.utils.config import Configurator
from jbutils import jbutils

# from .config import Configurator
sample_files_1 = ["init.json", "test.yaml"]
sample_files_2 = {
    "common": {
        "tools": {
            "test1.json": {"a": 5, "b": "b", "c": {"c1": 1, "c2": False}},
            "test2.yaml": {"a": 10, "b": "B", "c": [{"c1": 2, "c2": True}]},
        }
    },
    "init.yaml": {"a": 15, "b": "test", "c": {"c1": 3, "c2": "true"}},
}


@dataclass
class TestC:
    c1: int = -1
    c2: bool = True


@dataclass
class Test1:
    a: int = -1
    b: str = ""
    c: TestC | dict = None

    def __post_init__(self) -> None:
        if isinstance(self.c, dict):
            self.c = TestC(**self.c)


def main() -> None:
    cfg = Configurator(app_name="cfgtest")
    dpath = "saved_data.test3.yaml"
    embed(
        globals=globals(), locals=locals(), history_filename="jbutils_cli.history"
    )


if __name__ == "__main__":
    main()
